@@ -1,7 +1,17 @@
 .do_by_variable_table <- function(dataset, class_variable, input.location){
+  
   df <- table(eval(parse(text = paste0("dataset$", class_variable))),
                     dataset$location_mode)
   
+  if(nrow(df) == 0) return()
+  
+  if("show.uncertainty" %in% input.location){
+    dataset$fuzzy.sum <- factor(dataset$fuzzy.sum, levels = c(0, 1, 2, 3), 
+                                labels = c("exact", "linear", "planar", "volume"))
+    df <- table(eval(parse(text = paste0("dataset$", class_variable))),
+                dataset$fuzzy.sum)
+  }
+  
   if(nrow(df) > 1 & ncol(df) > 1){
     df <- as.matrix(df)
     df <- df[order(df[,1], decreasing = T), ]
@@ -19,6 +29,7 @@
   }
   
   colnames(df) <- sapply(colnames(df), .term_switcher)
+  rownames(df)[nrow(df)] <- .term_switcher("total")
   
   df
 }
\ No newline at end of file
